/*****************************************************************************/
/*                                                                           */
/*   draw.cpp                                                                */
/*   www.xbdev.net                                                           */
/*                                                                           */
/*****************************************************************************/


#include <windows.h>
#include <assert.h>
#include <d3d9.h>
#include <d3dx9.h>
#include <d3dx9mesh.h>  


#pragma comment(lib, "d3dx9.lib")
#pragma comment(lib, "d3d9.lib")

#include <stdio.h>  //sprintf(..)

#include "dxdbg.h"


#define SZ_FX_FILE					"media//bump.fx"

#define SZ_X_FILE					"media//tank.x"

#define SZ_TEXTURE_FILE0			"media//rockwall.png"
#define SZ_TEXTURE_FILE1			"media//rockwall_normal.png"



extern LPDIRECT3DDEVICE9			g_pd3dDevice;
extern HWND							g_hWnd;

extern float						g_fSpinX;
extern float						g_fSpinY;
extern float						g_fZoom;


static ID3DXEffect*					g_pEffect				= NULL;
static LPDIRECT3DVERTEXDECLARATION9 g_pVertexDeclaration	= NULL;

static IDirect3DTexture9*			g_pTexture0;
static IDirect3DTexture9*			g_pTexture1;

static LPD3DXBUFFER					g_pMtrlBuffer = NULL;
static DWORD						g_dwMaterials;
static LPD3DXMESH					g_pMesh;


struct MESH_VERTEX
{
	D3DXVECTOR3 position; 
	D3DXVECTOR3 normal; 
	D3DXVECTOR2 tex0;		// texture coords
	D3DXVECTOR3 tangent;	// tangent vector (basis vector s)
	D3DXVECTOR3 binormal;	// binormal vector (basis vector t)
}; 


bool LoadShader();
bool LoadMesh();
bool LoadTextures();
HRESULT MyComputeTangents( ID3DXMesh * pMesh );
HRESULT MyComputeNormals( ID3DXMesh * pMesh );



const D3DVERTEXELEMENT9 declaration[] = 
{
	{ 0, 0  * sizeof(float),D3DDECLTYPE_FLOAT3,   D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 0 },  // pos
	{ 0, 3  * sizeof(float),D3DDECLTYPE_FLOAT3,   D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_NORMAL,   0 },  // normal
	{ 0, 6  * sizeof(float),D3DDECLTYPE_FLOAT2,   D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 0 },  // tex0
	{ 0, 8  * sizeof(float),D3DDECLTYPE_FLOAT3,   D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TANGENT,  0 },  // tangent
	{ 0, 11 * sizeof(float),D3DDECLTYPE_FLOAT3,   D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_BINORMAL, 0 },  // binormal
	D3DDECL_END()
};


bool LoadMesh()
{
	ID3DXMesh * pMesh = NULL;
	HRESULT hr = S_OK;							// Error checking variable

	hr |= 
	// File Name ---------+
    //                    |
    //                   \|/
    //                    |
    D3DXLoadMeshFromX( SZ_X_FILE,				// pFilename
						D3DXMESH_SYSTEMMEM,		// Options
						g_pd3dDevice,			// pD3DDevice
                        NULL,					// ppAdjacency
						&g_pMtrlBuffer,			// ppMaterials
						NULL,					// ppEffectInstances
						&g_dwMaterials,			// pNumMaterials
						&pMesh);				// ppMesh

	hr |= 
	pMesh->CloneMesh( D3DXMESH_SYSTEMMEM,
						declaration,
						g_pd3dDevice,
						&g_pMesh );
	pMesh->Release();

#if(0)
	hr |= 
	D3DXComputeNormals(g_pMesh,					// Mesh
						NULL);					// pAdjacency

	hr |= 
	D3DXComputeTangent(g_pMesh,					// Mesh
						0,						// TexStageIndex
						0,						// TangentIndex
						0,						// BinormIndex
						FALSE,					// Wrap
						NULL);					// pAdjacency
#endif

	MyComputeNormals( g_pMesh );

	MyComputeTangents( g_pMesh );

	if( hr!= S_OK )
	{
		MessageBox(0, "Failed in LoadMesh()", "Load Error", 0);
		return false;
	}

	return true;
}


void CalculateNormals(	      MESH_VERTEX *pVertex,
						int   nVertices, 
						const WORD *pTriangle,
						int   nTriangles)
{
    D3DXVECTOR3 *norm = new D3DXVECTOR3[nVertices];
	memset(norm, 0, sizeof(D3DXVECTOR3) * nVertices);

    for (long a = 0; a < nTriangles; a++)
    {
        long i1 = pTriangle[0];
        long i2 = pTriangle[1];
        long i3 = pTriangle[2];
        
		D3DXVECTOR3 v1 = pVertex[i1].position;
        D3DXVECTOR3 v2 = pVertex[i2].position;
		D3DXVECTOR3 v3 = pVertex[i3].position;
        
		D3DXVECTOR3 vA = v2 - v1;
		D3DXVECTOR3 vB = v3 - v1;

		D3DXVECTOR3 vNorm;
		D3DXVec3Cross(&vNorm, &vA, &vB);

		D3DXVec3Normalize(&vNorm, &vNorm);

        norm[i1] += vNorm;
		norm[i2] += vNorm;
		norm[i3] += vNorm;

        pTriangle+=3;
    }
    
	for (int i = 0; i < nVertices; ++i)
	{
		D3DXVec3Normalize(&norm[i], &norm[i]);
		pVertex[i].normal = norm[i];
	}

	delete[] norm;
}


HRESULT MyComputeNormals( ID3DXMesh * pMesh )
{
	/*
	struct MESH_VERTEX
	{
	D3DXVECTOR3 position; 
	D3DXVECTOR3 normal; 
	D3DXVECTOR2 tex0;		// texture coords
	D3DXVECTOR3 tangent;	// tangent vector (basis vector s)
	D3DXVECTOR3 binormal;	// binormal vector (basis vector t)
	};
	*/

	MESH_VERTEX *pMeshVertices;
	WORD * pMeshIndices;

	pMesh->LockIndexBuffer(D3DLOCK_NOSYSLOCK, (void**)&pMeshIndices);
	pMesh->LockVertexBuffer(D3DLOCK_NOSYSLOCK, (void**)&pMeshVertices);


	try{

	CalculateNormals(	pMeshVertices,
						g_pMesh->GetNumVertices(),
						pMeshIndices,
						g_pMesh->GetNumFaces());
	}
	catch(...){
		MessageBox(0, "Failed in MyComputeNormals()\n"
				          "Possible Errors - Locking Index/Vertex Bufs, Calc Normals Algorithm", 
						  "Load Error", 0);
		return S_FALSE;
	}
	pMesh->UnlockVertexBuffer();
	pMesh->UnlockIndexBuffer();

	return S_OK;
}


void CalculateTangents( MESH_VERTEX *pVertex, 
						int nVertices, 
						const WORD *pTriangle, 
						int nTriangles)
{
	D3DXVECTOR3 *tan = new D3DXVECTOR3[nVertices];
	D3DXVECTOR3 *bin = new D3DXVECTOR3[nVertices];

	memset(tan, 0, sizeof(D3DXVECTOR3) * nVertices);
	memset(bin, 0, sizeof(D3DXVECTOR3) * nVertices);

	// Calculate tangents and binormals.
	for (int i = 0; i < nTriangles; ++i)
	{
		int i1 = pTriangle[i * 3];
		int i2 = pTriangle[i * 3 + 1];
		int i3 = pTriangle[i * 3 + 2];

		D3DXVECTOR3 v1 = pVertex[i1].position;
		D3DXVECTOR3 v2 = pVertex[i2].position;
		D3DXVECTOR3 v3 = pVertex[i3].position;

		D3DXVECTOR2 w1 = pVertex[i1].tex0;
		D3DXVECTOR2 w2 = pVertex[i2].tex0;
		D3DXVECTOR2 w3 = pVertex[i3].tex0;

		float x1 = v2.x - v1.x;
		float y1 = v2.y - v1.y;
		float z1 = v2.z - v1.z;
		float x2 = v3.x - v1.x;
		float y2 = v3.y - v1.y;
		float z2 = v3.z - v1.z;

		float s1 = w2.x - w1.x;
		float t1 = w2.y - w1.y;
		float s2 = w3.x - w1.x;
		float t2 = w3.y - w1.y;

		float r = 1.0f / (s1 * t2 - s2 * t1);

		D3DXVECTOR3 sdir((t2 * x1 - t1 * x2) * r,
		(t2 * y1 - t1 * y2) * r,
		(t2 * z1 - t1 * z2) * r);

		D3DXVECTOR3 tdir((s1 * x2 - s2 * x1) * r,
		(s1 * y2 - s2 * y1) * r,
		(s1 * z2 - s2 * z1) * r);

		tan[i1] += sdir;
		tan[i2] += sdir;
		tan[i3] += sdir;

		bin[i1] += tdir;
		bin[i2] += tdir;
		bin[i3] += tdir;
	}

	// Save our tangents and binormals.
	for (int i = 0; i < nVertices; ++i)
	{
		D3DXVec3Normalize(&tan[i], &tan[i]);
		pVertex[i].tangent = tan[i];

		D3DXVec3Normalize(&bin[i], &bin[i]);
		pVertex[i].binormal = bin[i];
	}

	delete[] tan;
	delete[] bin;
}


HRESULT MyComputeTangents( ID3DXMesh * pMesh )
{
	/*
	struct MESH_VERTEX
	{
	D3DXVECTOR3 position; 
	D3DXVECTOR3 normal; 
	D3DXVECTOR2 tex0;		// texture coords
	D3DXVECTOR3 tangent;	// tangent vector (basis vector s)
	D3DXVECTOR3 binormal;	// binormal vector (basis vector t)
	};
	*/

	MESH_VERTEX *pMeshVertices;
	WORD * pMeshIndices;

	pMesh->LockIndexBuffer(D3DLOCK_NOSYSLOCK, (void**)&pMeshIndices);
	pMesh->LockVertexBuffer(D3DLOCK_NOSYSLOCK, (void**)&pMeshVertices);


	try{

	CalculateTangents(	pMeshVertices,
						g_pMesh->GetNumVertices(),
						pMeshIndices,
						g_pMesh->GetNumFaces());
	}
	catch(...){
		MessageBox(0, "Failed in MyComputeTangent()\n"
				          "Possible Errors - Locking Index/Vertex Bufs, Calc Tangent Algorithm", 
						  "Load Error", 0);
		return S_FALSE;
	}
	pMesh->UnlockVertexBuffer();
	pMesh->UnlockIndexBuffer();

	return S_OK;
}




bool LoadTextures()
{
	HRESULT hr = S_OK;							// Error checking variable

	hr |= 
	D3DXCreateTextureFromFile(g_pd3dDevice, 
                              SZ_TEXTURE_FILE0, 
                              &g_pTexture0);

	hr |=
	D3DXCreateTextureFromFile(g_pd3dDevice, 
                              SZ_TEXTURE_FILE1, 
                              &g_pTexture1);

	if( hr!= S_OK )
	{
		MessageBox(0, "Failed in LoadTextures()", "Load Error", 0);
		return false;
	}

	return true;
}

bool Create()
{
	bool bOkay = true;
	
	bOkay &= LoadShader();

	bOkay &= LoadMesh();

	bOkay &= LoadTextures();

	return bOkay;
}//End Create(..)


bool Release()
{
	g_pMesh->Release();
	g_pMtrlBuffer->Release();

	g_pTexture0->Release();
	g_pTexture1->Release();

	if( g_pEffect )
	{
		g_pEffect->Release();
		g_pVertexDeclaration->Release();
	}

	return true;
}// End Release()


bool LoadShader()
{
	LPD3DXBUFFER pBufferErrors;
	DWORD dwShaderFlags = 0;

	HRESULT hr =									// Return Error Checking
		
	D3DXCreateEffectFromFile(	g_pd3dDevice,		// pDevice
								SZ_FX_FILE,			// pSrcFile
								NULL,				// pDefines
								NULL,				// pInclude
								dwShaderFlags,		// Flags
								NULL,				// pPool
								&g_pEffect,			// ppEffect
								&pBufferErrors);	// ppCompilationErrors

	if(FAILED(hr) )
	{
		LPVOID pCompileErrors = pBufferErrors->GetBufferPointer(); 
		MessageBox(NULL, (const char*)pCompileErrors, "Compile Error", 
		MB_OK|MB_ICONEXCLAMATION);

		return false;
	}

	g_pd3dDevice->CreateVertexDeclaration( declaration, &g_pVertexDeclaration );
	g_pd3dDevice->SetVertexDeclaration( g_pVertexDeclaration );

	return true;
}// End LoadShader(..)




void SetShaderConstants(float x, float y, float z)
// Setting Shader Constants
{		
	D3DXMATRIX matTrans;
	D3DXMATRIX matRot;
    D3DXMATRIX matProj,matIWorld,matView,matWorld;

	D3DXMatrixTranslation( &matTrans, x, y, z + g_fZoom ); 
	

	D3DXMatrixRotationYawPitchRoll( &matRot, D3DXToRadian(g_fSpinX), 
										     D3DXToRadian(g_fSpinY), 
												0.0f );
	
	D3DXMatrixPerspectiveFovLH( &matProj, D3DXToRadian( 45.0f ), 
                            640.0f / 480.0f, 0.1f, 500.0f );

	matWorld = matRot * matTrans;

	D3DXMatrixIdentity( &matView ); // This sample is not really making use of a view matrix

	D3DXMATRIX worldViewProj = matWorld * matView * matProj;

	D3DXMATRIX matWorldI = matWorld;
	D3DXMatrixTranspose(&matWorldI, &matWorldI);

	g_pEffect->SetMatrix("matWorldViewProj", &worldViewProj);
	g_pEffect->SetMatrix("matWorld",  &matWorld);
	g_pEffect->SetMatrix("matWorldI", &matWorldI);

}//End SetShaderConstants(..)






void RenderShape()
{
	DWORD iPass = 0;
	UINT cPasses = 0;

	SetShaderConstants(0,0,0);

	g_pEffect->SetTechnique("BumpEffect");
	g_pd3dDevice->SetVertexDeclaration( g_pVertexDeclaration );

	g_pEffect->SetTexture("texture0", g_pTexture0);
	g_pEffect->SetTexture("texture1", g_pTexture1);

	g_pEffect->Begin(&cPasses, 0);	
	g_pEffect->BeginPass(0);

	g_pMesh->DrawSubset(0);
	
	//End of our Effect Render
	g_pEffect->EndPass();
	g_pEffect->End();
}// End RenderShape(..)



/***************************************************************************/
/*                                                                         */
/* render()                                                                */
/* Our main render loop, which gets called over and over agian to do our   */
/* drawing...clears the screen, draws the data, then presents it..         */
/*                                                                         */
/***************************************************************************/
bool Render()
{

#if(0)
	g_pd3dDevice->SetRenderState( D3DRS_FILLMODE, D3DFILL_WIREFRAME );
#else
	g_pd3dDevice->SetRenderState( D3DRS_FILLMODE, D3DFILL_SOLID );
#endif

	g_pd3dDevice->Clear( 0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, 0x88888800, 1.0f, 0 );
    g_pd3dDevice->BeginScene();
	
	RenderShape();


	TriDisplay(g_pd3dDevice, g_pMesh->GetNumFaces() );
	FPSDisplay(g_pd3dDevice);
    g_pd3dDevice->EndScene();
	g_pd3dDevice->Present( NULL, NULL, NULL, NULL );


	return true;

}//End Render()
